"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.iterateElements = void 0;
const tslib_1 = require("tslib");
const iter_tools_1 = require("iter-tools");
const create_array_equal_1 = tslib_1.__importDefault(require("create-array-equal"));
const special_character_1 = tslib_1.__importDefault(require("../utils/special-character"));
const digit_1 = tslib_1.__importDefault(require("../utils/digit"));
const sequence_tests_1 = require("../utils/sequence-tests");
const call_maybe_function_1 = tslib_1.__importDefault(require("../utils/call-maybe-function"));
const { Start, StartFollow, End, EndOfLine } = special_character_1.default;
const { Zero } = digit_1.default;
const arrayEqual = create_array_equal_1.default();
const RESET = [Start, StartFollow, Zero, End];
async function* iterateElements(splitter) {
    const { data, prefix, suffix } = splitter;
    let leadingCharacters = Array();
    let nextLeadingCharacters = Array();
    let currentEscape = [];
    let isInEscape = false;
    let currentLine = [];
    const createYieldValue = () => ({
        format: Array.from(leadingCharacters),
        reset: leadingCharacters.length ? RESET : [],
        main: currentLine,
        prefix: call_maybe_function_1.default(prefix, { splitter, currentLine, leadingCharacters }),
        suffix: call_maybe_function_1.default(suffix, { splitter, currentLine, leadingCharacters }),
    });
    function pushCurrentLine(char) {
        currentLine = [...currentLine, char];
    }
    function pushCurrentEscape(char) {
        currentEscape = [...currentEscape, char];
    }
    for await (const char of data) {
        // Handle special characters
        switch (char) {
            case Start:
                currentEscape = [char];
                pushCurrentLine(char);
                break;
            case StartFollow:
                if (arrayEqual(currentEscape, [Start])) {
                    currentEscape = [Start, StartFollow];
                    isInEscape = true;
                }
                pushCurrentLine(char);
                break;
            case End:
                if (isInEscape) {
                    isInEscape = false;
                    // If meet '\e[m', '\e[0m', '\e[00m'...
                    // NOTE: No need for 'm' suffix
                    if (sequence_tests_1.isResetSequence(iter_tools_1.slice({ start: 2 }, currentEscape))) {
                        nextLeadingCharacters = []; // empty newEscape if meet '\e[0m'
                    }
                    else {
                        nextLeadingCharacters.push([...currentEscape, End]); // otherwise, add '\e[Xm'
                    }
                    currentEscape = [];
                }
                pushCurrentLine(char);
                break;
            // Restore SGR state from previous line
            case EndOfLine:
                yield createYieldValue();
                currentLine = [];
                leadingCharacters = [...nextLeadingCharacters];
                nextLeadingCharacters = [];
                break;
            // When the character is not special
            default:
                if (isInEscape)
                    pushCurrentEscape(char);
                pushCurrentLine(char);
        }
    }
    yield createYieldValue();
}
exports.iterateElements = iterateElements;
exports.default = iterateElements;
