import { slice } from "iter-tools/es5/index.mjs";
import createArrayEqual from 'create-array-equal';
import SpecialCharacter from "../utils/special-character.mjs";
import Digit from "../utils/digit.mjs";
import { isResetSequence } from "../utils/sequence-tests.mjs";
import callMaybeFunction from "../utils/call-maybe-function.mjs";
const {
  Start,
  StartFollow,
  End,
  EndOfLine
} = SpecialCharacter;
const {
  Zero
} = Digit;
const arrayEqual = createArrayEqual();
const RESET = [Start, StartFollow, Zero, End];
export async function* iterateElements(splitter) {
  const {
    data,
    prefix,
    suffix
  } = splitter;
  let leadingCharacters = Array();
  let nextLeadingCharacters = Array();
  let currentEscape = [];
  let isInEscape = false;
  let currentLine = [];

  const createYieldValue = () => ({
    format: Array.from(leadingCharacters),
    reset: leadingCharacters.length ? RESET : [],
    main: currentLine,
    prefix: callMaybeFunction(prefix, {
      splitter,
      currentLine,
      leadingCharacters
    }),
    suffix: callMaybeFunction(suffix, {
      splitter,
      currentLine,
      leadingCharacters
    })
  });

  function pushCurrentLine(char) {
    currentLine = [...currentLine, char];
  }

  function pushCurrentEscape(char) {
    currentEscape = [...currentEscape, char];
  }

  for await (const char of data) {
    // Handle special characters
    switch (char) {
      case Start:
        currentEscape = [char];
        pushCurrentLine(char);
        break;

      case StartFollow:
        if (arrayEqual(currentEscape, [Start])) {
          currentEscape = [Start, StartFollow];
          isInEscape = true;
        }

        pushCurrentLine(char);
        break;

      case End:
        if (isInEscape) {
          isInEscape = false; // If meet '\e[m', '\e[0m', '\e[00m'...
          // NOTE: No need for 'm' suffix

          if (isResetSequence(slice({
            start: 2
          }, currentEscape))) {
            nextLeadingCharacters = []; // empty newEscape if meet '\e[0m'
          } else {
            nextLeadingCharacters.push([...currentEscape, End]); // otherwise, add '\e[Xm'
          }

          currentEscape = [];
        }

        pushCurrentLine(char);
        break;
      // Restore SGR state from previous line

      case EndOfLine:
        yield createYieldValue();
        currentLine = [];
        leadingCharacters = [...nextLeadingCharacters];
        nextLeadingCharacters = [];
        break;
      // When the character is not special

      default:
        if (isInEscape) pushCurrentEscape(char);
        pushCurrentLine(char);
    }
  }

  yield createYieldValue();
}
export default iterateElements;